@@ -13,6 +13,6 @@
 #include "decoder_util.h"
 
 /* TODO: temporary allow to change to new style model keys */
-#define _X(n, o) (decoder->new_model_keys ? (n) : (o))
+#define _X(n, o) (decoder->old_model_keys ? (o) : (n))
 
 #endif /* INCLUDE_DECODER_H_ */
