@@ -16,6 +16,8 @@
 #include <string.h>
 #include <math.h>
 
+#include "r_util.h"
+
 static uint16_t scaled_squares[256];
 
 /// precalculate lookup table for envelope detection.
@@ -125,7 +127,7 @@ void baseband_low_pass_filter(uint16_t const *x_buf, int16_t *y_buf, uint32_t le
     static int const b[FILTER_ORDER + 1] = {FIX(0.07296), FIX(0.07296)};
 
     unsigned long i;
-    // Fixme: Will Segmentation Fault if len < FILTERORDER
+    // TODO: Will Segmentation Fault if len < FILTERORDER
 
     /* Calculate first sample */
     y_buf[0] = ((a[1] * state->y[0] >> 1) + (b[0] * x_buf[0] >> 1) + (b[1] * state->x[0] >> 1)) >> (F_SCALE - 1);
@@ -149,7 +151,7 @@ void baseband_low_pass_filter(uint16_t const *x_buf, int16_t *y_buf, uint32_t le
     @param x Denominator (real value of complex vector)
     @return angle in radians (Pi equals INT16_MAX)
 */
-static int16_t atan2_int16(int16_t y, int16_t x)
+static int16_t atan2_int16(int32_t y, int32_t x)
 {
     static int32_t const I_PI_4 = INT16_MAX/4;      // M_PI/4
     static int32_t const I_3_PI_4 = 3*INT16_MAX/4;  // 3*M_PI/4
@@ -157,6 +159,8 @@ static int16_t atan2_int16(int16_t y, int16_t x)
     int32_t const abs_y = abs(y);
     int32_t angle;
 
+    if (!x && !y) return 0; // We would get 8191 with the code below
+
     if (x >= 0) {    // Quadrant I and IV
         int32_t denom = (abs_y + x);
         if (denom == 0) denom = 1;  // Prevent divide by zero
@@ -255,6 +259,7 @@ static int32_t atan2_int32(int32_t y, int32_t x)
 /// for evaluation.
 void baseband_demod_FM_cs16(int16_t const *x_buf, int16_t *y_buf, unsigned long num_samples, demodfm_state_t *state, unsigned fpdm)
 {
+    UNUSED(fpdm);
     ///  [b,a] = butter(1, 0.1) -> 3x tau (95%) ~10 samples
     //static int const alp[2] = {FIX32(1.00000), FIX32(0.72654)};
     //static int const blp[2] = {FIX32(0.13673), FIX32(0.13673)};
