@@ -1,5 +1,11 @@
 /*** @file
     AlectoV1 Weather Sensor protocol.
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
 */
 /** @fn int alectov1_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 AlectoV1 Weather Sensor decoder.
@@ -100,7 +106,6 @@ int alecto_checksum(r_device *decoder, bitrow_t *bb)
     return 1;
 }
 
-
 static uint8_t bcd_decode8(uint8_t x)
 {
     return ((x & 0xF0) >> 4) * 10 + (x & 0x0F);
@@ -110,10 +115,8 @@ static int alectov1_callback(r_device *decoder, bitbuffer_t *bitbuffer)
 {
     bitrow_t *bb = bitbuffer->bb;
     uint8_t *b = bitbuffer->bb[1];
-    int16_t temp;
-    uint8_t humidity;
-    int ret;
-
+    int temp_raw, humidity;
+    float temp_c;
     data_t *data;
     unsigned bits = bitbuffer->bits_per_row[1];
 
@@ -125,16 +128,15 @@ static int alectov1_callback(r_device *decoder, bitbuffer_t *bitbuffer)
             || bb[5][0] == 0 || bb[5][1] == 0)
         return DECODE_ABORT_EARLY;
 
-    ret = alecto_checksum(decoder, bb);
-    if (ret <= 0)
+    if (alecto_checksum(decoder, bb) <= 0)
         return DECODE_FAIL_MIC;
 
     int battery_low = (b[1] & 0x80) >> 7;
     int msg_type    = (b[1] & 0x60) >> 5;
-    int button      = (b[1] & 0x10) >> 4;
+    //int button      = (b[1] & 0x10) >> 4;
     int msg_rain    = (b[1] & 0x0f) == 0x0c;
-    int msg_wind    = (b[1] & 0x0f) == 0x08 && b[2] == 0;
-    int msg_gust    = (b[1] & 0x0e) == 0x0e;
+    //int msg_wind    = (b[1] & 0x0f) == 0x08 && b[2] == 0;
+    //int msg_gust    = (b[1] & 0x0e) == 0x0e;
     int channel     = (b[0] & 0xc) >> 2;
     int sensor_id   = reverse8(b[0]);
 
@@ -155,6 +157,7 @@ static int alectov1_callback(r_device *decoder, bitbuffer_t *bitbuffer)
             double gust = reverse8(bb[5 + skip][3]);
             int direction = (reverse8(bb[5 + skip][2]) << 1) | (bb[5 + skip][1] & 0x1);
 
+            /* clang-format off */
             data = data_make(
                     "model",          "",           DATA_STRING, _X("AlectoV1-Wind","AlectoV1 Wind Sensor"),
                     "id",             "House Code", DATA_INT,    sensor_id,
@@ -165,6 +168,7 @@ static int alectov1_callback(r_device *decoder, bitbuffer_t *bitbuffer)
                     _X("wind_dir_deg","wind_direction"),   "Wind Direction",   DATA_INT, direction,
                     "mic",           "Integrity",   DATA_STRING,    "CHECKSUM",
                     NULL);
+            /* clang-format on */
             decoder_output_data(decoder, data);
             return 1;
         }
@@ -174,6 +178,7 @@ static int alectov1_callback(r_device *decoder, bitbuffer_t *bitbuffer)
         // Rain sensor
         double rain_mm = ((reverse8(b[3]) << 8) | reverse8(b[2])) * 0.25F;
 
+        /* clang-format off */
         data = data_make(
                 "model",         "",           DATA_STRING, _X("AlectoV1-Rain","AlectoV1 Rain Sensor"),
                 "id",            "House Code", DATA_INT,    sensor_id,
@@ -182,6 +187,7 @@ static int alectov1_callback(r_device *decoder, bitbuffer_t *bitbuffer)
                 _X("rain_mm","rain_total"),    "Total Rain", DATA_FORMAT, "%.02f mm", DATA_DOUBLE, rain_mm,
                 "mic",           "Integrity",  DATA_STRING,    "CHECKSUM",
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -191,23 +197,23 @@ static int alectov1_callback(r_device *decoder, bitbuffer_t *bitbuffer)
             && bb[4][0] == bb[5][0] && bb[5][0] == bb[6][0]
             && (bb[3][4] & 0xf) == 0 && (bb[5][4] & 0xf) == 0) {
         //static char * temp_states[4] = {"stable", "increasing", "decreasing", "invalid"};
-        temp = (int16_t) ((uint16_t) (reverse8(b[1]) >> 4) | (reverse8(b[2]) << 4));
-        if ((temp & 0x800) != 0) {
-            temp |= 0xf000;
-        }
+        temp_raw = (int16_t)((reverse8(b[1]) & 0xf0) | (reverse8(b[2]) << 8)); // sign-extend
+        temp_c   = (temp_raw >> 4) * 0.1f;
         humidity = bcd_decode8(reverse8(b[3]));
-        if (humidity>100)
-            return DECODE_FAIL_SANITY;//extra detection false positive!! prologue is also 36bits and sometimes detected as alecto
+        if (humidity > 100)
+            return DECODE_FAIL_SANITY; // detect false positive, prologue is also 36bits and sometimes detected as alecto
 
+        /* clang-format off */
         data = data_make(
                 "model",         "",            DATA_STRING, _X("AlectoV1-Temperature","AlectoV1 Temperature Sensor"),
                 "id",            "House Code",  DATA_INT,    sensor_id,
                 "channel",       "Channel",     DATA_INT,    channel,
                 "battery",       "Battery",     DATA_STRING, battery_low ? "LOW" : "OK",
-                "temperature_C", "Temperature", DATA_FORMAT, "%.02f C", DATA_DOUBLE, (float) temp / 10.0F,
+                "temperature_C", "Temperature", DATA_FORMAT, "%.02f C", DATA_DOUBLE, temp_c,
                 "humidity",      "Humidity",    DATA_FORMAT, "%u %%",   DATA_INT, humidity,
                 "mic",           "Integrity",   DATA_STRING,    "CHECKSUM",
                 NULL);
+        /* clang-format on */
         decoder_output_data(decoder, data);
         return 1;
     }
@@ -227,11 +233,11 @@ static char *output_fields[] = {
     "wind_speed", // TODO: remove this
     "wind_gust", // TODO: remove this
     "wind_direction", // TODO: remove this
-    "wind_avg_km_h",
-    "wind_max_km_h",
+    "wind_avg_m_s",
+    "wind_max_m_s",
     "wind_dir_deg",
     "mic",
-    NULL
+    NULL,
 };
 
 r_device alectov1 = {
@@ -242,6 +248,5 @@ r_device alectov1 = {
     .gap_limit      = 7000,
     .reset_limit    = 10000,
     .decode_fn      = &alectov1_callback,
-    .disabled       = 0,
-    .fields         = output_fields
+    .fields         = output_fields,
 };
