@@ -29,7 +29,8 @@ Power calculations come from Nathaniel Elijah's program EfergyRPI_001.
 
 #include "decoder.h"
 
-static int efergy_e2_classic_callback(r_device *decoder, bitbuffer_t *bitbuffer) {
+static int efergy_e2_classic_callback(r_device *decoder, bitbuffer_t *bitbuffer)
+{
     unsigned num_bits = bitbuffer->bits_per_row[0];
     uint8_t *bytes = bitbuffer->bb[0];
     data_t *data;
@@ -54,6 +55,7 @@ static int efergy_e2_classic_callback(r_device *decoder, bitbuffer_t *bitbuffer)
         }
     }
 
+
     // Sometimes pulses and gaps are mixed up. If this happens, invert
     // all bytes to get correct interpretation.
     if (bytes[0] & 0xf0) {
@@ -62,7 +64,17 @@ static int efergy_e2_classic_callback(r_device *decoder, bitbuffer_t *bitbuffer)
         }
     }
 
+    int zero_count = 0;
+    for (int i=0; i<8; i++) {
+        if (bytes[i] == 0)
+            zero_count++;
+    }
+    if (zero_count++ > 5)
+        return DECODE_FAIL_SANITY;  // too many Null bytes
+
+
     unsigned checksum = add_bytes(bytes, 7);
+
     if (checksum == 0) {
         return DECODE_FAIL_SANITY; // reduce false positives
     }
@@ -102,6 +114,7 @@ static char *output_fields[] = {
         "current",
         "interval",
         "learn",
+        "mic",
         NULL,
 };
 
