@@ -1,15 +1,17 @@
-/* Generic temperature sensor 1
- *
- * Copyright (C) 2015 Alexandre Coffignal
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- */
-/*
-10 24 bits frames
+/** @file
+    Generic temperature sensor 1.
+
+    Copyright (C) 2015 Alexandre Coffignal
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+*/
+/**
+Generic temperature sensor 1.
+
+10 24 bits frames:
 
     IIIIIIII BBTTTTTT TTTTTTTT
 
@@ -20,10 +22,11 @@
 
 #include "decoder.h"
 
-static int generic_temperature_sensor_callback(r_device *decoder, bitbuffer_t *bitbuffer) {
+static int generic_temperature_sensor_callback(r_device *decoder, bitbuffer_t *bitbuffer)
+{
     data_t *data;
     uint8_t *b = bitbuffer->bb[1];
-    int i, device, battery;
+    int i, device, battery, temp_raw;
     float temp_f;
 
     for (i = 1; i < 10; i++) {
@@ -41,7 +44,8 @@ static int generic_temperature_sensor_callback(r_device *decoder, bitbuffer_t *b
 
     device  = (b[0]);
     battery = (b[1] & 0xF0) >> 4;
-    temp_f  = (float)((signed short)(((b[1] & 0x3f) * 256 + b[2]) << 2)) / 160.0;
+    temp_raw = (int16_t)(((b[1] & 0x3f) << 10) | (b[2] << 2));
+    temp_f  = (temp_raw >> 4) * 0.1f;
 
     /* clang-format off */
     data = data_make(
@@ -57,21 +61,21 @@ static int generic_temperature_sensor_callback(r_device *decoder, bitbuffer_t *b
 }
 
 static char *output_fields[] = {
-    "model",
-    "id",
-    "battery",
-    "temperature_C",
-    NULL
+        "model",
+        "id",
+        "battery",
+        "temperature_C",
+        NULL,
 };
 
 r_device generic_temperature_sensor = {
-    .name          = "Generic temperature sensor 1",
-    .modulation    = OOK_PULSE_PPM,
-    .short_width   = 2000,
-    .long_width    = 4000,
-    .gap_limit     = 4800,
-    .reset_limit   = 10000,
-    .decode_fn     = &generic_temperature_sensor_callback,
-    .disabled      = 0,
-    .fields        = output_fields,
+        .name        = "Generic temperature sensor 1",
+        .modulation  = OOK_PULSE_PPM,
+        .short_width = 2000,
+        .long_width  = 4000,
+        .gap_limit   = 4800,
+        .reset_limit = 10000,
+        .decode_fn   = &generic_temperature_sensor_callback,
+        .disabled    = 0,
+        .fields      = output_fields,
 };
