@@ -53,7 +53,8 @@ static int honeywell_decode(r_device *decoder, bitbuffer_t *bitbuffer)
     int event;
     uint16_t crc_calculated;
     uint16_t crc;
-    int state;
+    int reed;
+    int contact;
     int heartbeat;
     int alarm;
     int tamper;
@@ -93,11 +94,12 @@ static int honeywell_decode(r_device *decoder, bitbuffer_t *bitbuffer)
         return DECODE_FAIL_MIC; // Not a valid packet
 
     event = b[3];
-    // decoded event bits: AATABHUU
+    // decoded event bits: CTRABHUU
     // NOTE: not sure if these apply to all device types
-    state       = (event & 0x80) >> 7;
-    alarm       = (event & 0xb0) >> 4;
+    contact     = (event & 0x80) >> 7;
     tamper      = (event & 0x40) >> 6;
+    reed        = (event & 0x20) >> 5;
+    alarm       = (event & 0x10) >> 4;
     battery_low = (event & 0x08) >> 3;
     heartbeat   = (event & 0x04) >> 2;
 
@@ -107,7 +109,9 @@ static int honeywell_decode(r_device *decoder, bitbuffer_t *bitbuffer)
             "id",           "", DATA_FORMAT, "%05x", DATA_INT, device_id,
             "channel",      "", DATA_INT,    channel,
             "event",        "", DATA_FORMAT, "%02x", DATA_INT, event,
-            "state",        "", DATA_STRING, state ? "open" : "closed",
+            "state",        "", DATA_STRING, contact ? "open" : "closed", // Ignore the reed switch legacy.
+            "contact_open", "", DATA_INT,    contact,
+            "reed_open",    "", DATA_INT,    reed,
             "alarm",        "", DATA_INT,    alarm,
             "tamper",       "", DATA_INT,    tamper,
             "battery_ok",   "", DATA_INT,    !battery_low,
@@ -125,6 +129,8 @@ static char *output_fields[] = {
         "channel",
         "event",
         "state",
+        "contact_open",
+        "reed_open",
         "alarm",
         "tamper",
         "battery_ok",
