@@ -1,6 +1,16 @@
+/** @file
+    Mebus 433.
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+*/
+
 #include "decoder.h"
 
-static int mebus433_callback(r_device *decoder, bitbuffer_t *bitbuffer) {
+static int mebus433_callback(r_device *decoder, bitbuffer_t *bitbuffer)
+{
     bitrow_t *bb = bitbuffer->bb;
     int16_t temp;
     int8_t  hum;
@@ -40,7 +50,7 @@ static int mebus433_callback(r_device *decoder, bitbuffer_t *bitbuffer) {
                 "battery",       "Battery",     DATA_STRING, battery ? "OK" : "LOW",
                 "unknown1",      "Unknown 1",   DATA_INT, unknown1,
                 "unknown2",      "Unknown 2",   DATA_INT, unknown2,
-                "temperature_C", "Temperature", DATA_FORMAT, "%.02f C", DATA_DOUBLE, temp / 10.0,
+                "temperature_C", "Temperature", DATA_FORMAT, "%.02f C", DATA_DOUBLE, temp * 0.1f,
                 "humidity",      "Humidity",    DATA_FORMAT, "%u %%", DATA_INT, hum,
                 NULL);
         decoder_output_data(decoder, data);
@@ -60,7 +70,7 @@ static char *output_fields[] = {
     "unknown2",
     "temperature_C",
     "humidity",
-    NULL
+    NULL,
 };
 
 r_device mebus433 = {
@@ -72,5 +82,5 @@ r_device mebus433 = {
     .reset_limit    = 6000,
     .decode_fn      = &mebus433_callback,
     .disabled       = 1, // add docs, tests, false positive checks and then reenable
-    .fields         = output_fields
+    .fields         = output_fields,
 };
