@@ -39,12 +39,28 @@ static int oregon_scientific_sl109h_callback(r_device *decoder, bitbuffer_t *bit
             continue; // DECODE_ABORT_LENGTH
 
         msg = bitbuffer->bb[row_index];
+
+        // No need to decode/extract values for simple test
+        // check id channel temperature humidity value not zero
+        if ( !msg[0] && !msg[1] && !msg[2] && !msg[3] ) {
+            if (decoder->verbose > 1) {
+                fprintf(stderr, "%s: DECODE_FAIL_SANITY data all 0x00\n", __func__);
+            }
+            continue; // DECODE_FAIL_SANITY
+        }
+
         chk = msg[0] >> 4;
 
         // align the channel "half nibble"
         bitbuffer_extract_bytes(bitbuffer, row_index, 2, b, 36);
         b[0] &= 0x3f;
 
+        // Prevent false positives from 'allzero'
+        // reject if Checksum channelhumidity and temperature are all zero
+        // No need to decode/extract values for simple test
+        if (chk == 0 && b[0] == 0 && b[1] == 0 && b[2] == 0)
+            continue; // DECODE_FAIL_SANITY
+
         sum = add_nibbles(b, 5) & 0xf;
         if (sum != chk) {
             if (decoder->verbose > 1)
