@@ -1,42 +1,49 @@
-/* Wireless Smoke & Heat Detector
- * Ningbo Siterwell Electronics  GS 558  Sw. V05  Ver. 1.3  on 433.885MHz
- * VisorTech RWM-460.f  Sw. V05, distributed by PEARL, seen on 433.674MHz
- *
- * A short wakeup pulse followed by a wide gap (11764 us gap),
- * followed by 24 data pulses and 2 short stop pulses (in a single bit width).
- * This is repeated 8 times with the next wakeup directly following
- * the preceding stop pulses.
- *
- * Bit width is 1731 us with
- * Short pulse: -___ 436 us pulse + 1299 us gap
- * Long pulse:  ---_ 1202 us pulse + 526 us gap
- * Stop pulse:  -_-_ 434us pulse + 434us gap + 434us pulse + 434us gap
- * = 2300 baud pulse width / 578 baud bit width
- *
- * 24 bits (6 nibbles):
- * - first 5 bits are unit number with bits reversed
- * - next 15(?) bits are group id, likely also reversed
- * - last 4 bits are always 0x3 (maybe hardware/protocol version)
- * Decoding will reverse the whole packet.
- * Short pulses are 0, long pulses 1, need to invert the demod output.
- *
- * Each device has it's own group id and unit number as well as a
- * shared/learned group id and unit number.
- * In learn mode the primary will offer it's group id and the next unit number.
- * The secondary device acknowledges pairing with 16 0x555555 packets
- * and copies the offered shared group id and unit number.
- * The primary device then increases it's unit number.
- * This means the primary will always have the same unit number as the
- * last learned secondary, weird.
- * Also you always need to learn from the same primary.
- *
- * Copyright (C) 2017 Christian W. Zuckschwerdt <zany@triq.net>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- */
+/** @file
+    Wireless Smoke & Heat Detector.
+
+    Copyright (C) 2017 Christian W. Zuckschwerdt <zany@triq.net>
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+*/
+/**
+Wireless Smoke & Heat Detector.
+
+Ningbo Siterwell Electronics  GS 558  Sw. V05  Ver. 1.3  on 433.885MHz
+VisorTech RWM-460.f  Sw. V05, distributed by PEARL, seen on 433.674MHz
+
+A short wakeup pulse followed by a wide gap (11764 us gap),
+followed by 24 data pulses and 2 short stop pulses (in a single bit width).
+This is repeated 8 times with the next wakeup directly following
+the preceding stop pulses.
+
+Bit width is 1731 us with
+Short pulse: -___ 436 us pulse + 1299 us gap
+Long pulse:  ---_ 1202 us pulse + 526 us gap
+Stop pulse:  -_-_ 434us pulse + 434us gap + 434us pulse + 434us gap
+= 2300 baud pulse width / 578 baud bit width
+
+24 bits (6 nibbles):
+- first 5 bits are unit number with bits reversed
+- next 15(?) bits are group id, likely also reversed
+- last 4 bits are always 0x3 (maybe hardware/protocol version)
+Decoding will reverse the whole packet.
+Short pulses are 0, long pulses 1, need to invert the demod output.
+
+Each device has it's own group id and unit number as well as a
+shared/learned group id and unit number.
+In learn mode the primary will offer it's group id and the next unit number.
+The secondary device acknowledges pairing with 16 0x555555 packets
+and copies the offered shared group id and unit number.
+The primary device then increases it's unit number.
+This means the primary will always have the same unit number as the
+last learned secondary, weird.
+Also you always need to learn from the same primary.
+
+*/
 
 #include "decoder.h"
 
@@ -76,6 +83,9 @@ static int smoke_gs558_callback(r_device *decoder, bitbuffer_t *bitbuffer)
     if (r < 0)
         return DECODE_ABORT_EARLY;
 
+    if (bitbuffer->bits_per_row[r] > 4*8)
+        return DECODE_ABORT_LENGTH;
+
     b = bitbuffer->bb[r];
 
     // if ((b[2] & 0x0f) != 0x03)
@@ -111,7 +121,7 @@ static char *output_fields[] = {
     "unit",
     "learn",
     "code",
-    NULL
+    NULL,
 };
 
 r_device smoke_gs558 = {
@@ -123,5 +133,5 @@ r_device smoke_gs558 = {
     .reset_limit    = 11764 * 1.2f, // Maximum gap size before End Of Message [us]
     .decode_fn      = &smoke_gs558_callback,
     .disabled       = 0,
-    .fields         = output_fields
+    .fields         = output_fields,
 };
