@@ -14,6 +14,7 @@
 #include "optparse.h"
 #include "util.h"
 #include "fatal.h"
+#include "r_util.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -25,6 +26,7 @@
 
 typedef struct mqtt_client {
     struct mg_send_mqtt_handshake_opts opts;
+    struct mg_connection *conn;
     int prev_status;
     char address[253 + 6 + 1]; // dns max + port
     char client_id[256];
@@ -35,7 +37,7 @@ typedef struct mqtt_client {
 static void mqtt_client_event(struct mg_connection *nc, int ev, void *ev_data)
 {
     // note that while shutting down the ctx is NULL
-    mqtt_client_t *ctx = (mqtt_client_t *)nc->mgr->user_data;
+    mqtt_client_t *ctx = (mqtt_client_t *)nc->user_data;
     // only valid in MG_EV_MQTT_ events
     struct mg_mqtt_message *msg = (struct mg_mqtt_message *)ev_data;
 
@@ -86,19 +88,17 @@ static void mqtt_client_event(struct mg_connection *nc, int ev, void *ev_data)
         if (ctx->prev_status == 0)
             fprintf(stderr, "MQTT Connection failed...\n");
         // reconnect
-        if (mg_connect(nc->mgr, ctx->address, mqtt_client_event) == NULL) {
+        struct mg_connect_opts opts = {.user_data = ctx};
+        ctx->conn = mg_connect_opt(nc->mgr, ctx->address, mqtt_client_event, opts);
+        if (!ctx->conn) {
             fprintf(stderr, "MQTT connect(%s) failed\n", ctx->address);
         }
         break;
     }
 }
 
-static struct mg_mgr *mqtt_client_init(char const *host, char const *port, char const *user, char const *pass, char const *client_id, int retain)
+static mqtt_client_t *mqtt_client_init(struct mg_mgr *mgr, char const *host, char const *port, char const *user, char const *pass, char const *client_id, int retain)
 {
-    struct mg_mgr *mgr = calloc(1, sizeof(*mgr));
-    if (!mgr)
-        FATAL_CALLOC("mqtt_client_init()");
-
     mqtt_client_t *ctx = calloc(1, sizeof(*ctx));
     if (!ctx)
         FATAL_CALLOC("mqtt_client_init()");
@@ -111,8 +111,7 @@ static struct mg_mgr *mqtt_client_init(char const *host, char const *port, char
     //ctx->timeout = 10000L;
     //ctx->cleansession = 1;
     strncpy(ctx->client_id, client_id, sizeof(ctx->client_id));
-
-    mg_mgr_init(mgr, ctx);
+    ctx->client_id[sizeof(ctx->client_id) - 1] = '\0';
 
     // if the host is an IPv6 address it needs quoting
     if (strchr(host, ':'))
@@ -120,35 +119,32 @@ static struct mg_mgr *mqtt_client_init(char const *host, char const *port, char
     else
         snprintf(ctx->address, sizeof(ctx->address), "%s:%s", host, port);
 
-    if (mg_connect(mgr, ctx->address, mqtt_client_event) == NULL) {
+    struct mg_connect_opts opts = {.user_data = ctx};
+    ctx->conn = mg_connect_opt(mgr, ctx->address, mqtt_client_event, opts);
+    if (!ctx->conn) {
         fprintf(stderr, "MQTT connect(%s) failed\n", ctx->address);
         exit(1);
     }
 
-    return mgr;
+    return ctx;
 }
 
-static int mqtt_client_poll(struct mg_mgr *mgr)
+static void mqtt_client_publish(mqtt_client_t *ctx, char const *topic, char const *str)
 {
-    return mg_mgr_poll(mgr, 0);
-}
+    if (!ctx->conn || !ctx->conn->proto_handler)
+        return;
 
-static void mqtt_client_publish(struct mg_mgr *mgr, char const *topic, char const *str)
-{
-    mqtt_client_t *ctx = (mqtt_client_t *)mgr->user_data;
     ctx->message_id++;
-
-    for (struct mg_connection *c = mg_next(mgr, NULL); c != NULL; c = mg_next(mgr, c)) {
-        if (c->proto_handler)
-            mg_mqtt_publish(c, topic, ctx->message_id, ctx->publish_flags, str, strlen(str));
-    }
+    mg_mqtt_publish(ctx->conn, topic, ctx->message_id, ctx->publish_flags, str, strlen(str));
 }
 
-static void mqtt_client_free(struct mg_mgr *mgr)
+static void mqtt_client_free(mqtt_client_t *ctx)
 {
-    free(mgr->user_data);
-    mgr->user_data = NULL;
-    mg_mgr_free(mgr);
+    if (ctx && ctx->conn) {
+        ctx->conn->user_data = NULL;
+        ctx->conn->flags |= MG_F_CLOSE_IMMEDIATELY;
+    }
+    free(ctx);
 }
 
 /* Helper */
@@ -167,7 +163,7 @@ static char *mqtt_sanitize_topic(char *topic)
 
 typedef struct {
     struct data_output output;
-    struct mg_mgr *mgr;
+    mqtt_client_t *mqc;
     char topic[256];
     char hostname[64];
     char *devices;
@@ -309,6 +305,7 @@ static char *expand_topic(char *topic, char const *format, data_t *data, char co
 // <prefix>[/type][/model][/subtype][/channel][/id]/battery: "OK"|"LOW"
 static void print_mqtt_data(data_output_t *output, data_t *data, char const *format)
 {
+    UNUSED(format);
     data_output_mqtt_t *mqtt = (data_output_mqtt_t *)output;
 
     char *orig = mqtt->topic + strlen(mqtt->topic); // save current topic
@@ -334,7 +331,7 @@ static void print_mqtt_data(data_output_t *output, data_t *data, char const *for
                 }
                 data_print_jsons(data, message, message_size);
                 expand_topic(mqtt->topic, mqtt->states, data, mqtt->hostname);
-                mqtt_client_publish(mqtt->mgr, mqtt->topic, message);
+                mqtt_client_publish(mqtt->mqc, mqtt->topic, message);
                 *mqtt->topic = '\0'; // clear topic
                 free(message);
             }
@@ -346,7 +343,7 @@ static void print_mqtt_data(data_output_t *output, data_t *data, char const *for
             char message[1024]; // we expect the biggest strings to be around 500 bytes.
             data_print_jsons(data, message, sizeof(message));
             expand_topic(mqtt->topic, mqtt->events, data, mqtt->hostname);
-            mqtt_client_publish(mqtt->mgr, mqtt->topic, message);
+            mqtt_client_publish(mqtt->mqc, mqtt->topic, message);
             *mqtt->topic = '\0'; // clear topic
         }
 
@@ -379,8 +376,9 @@ static void print_mqtt_data(data_output_t *output, data_t *data, char const *for
 
 static void print_mqtt_string(data_output_t *output, char const *str, char const *format)
 {
+    UNUSED(format);
     data_output_mqtt_t *mqtt = (data_output_mqtt_t *)output;
-    mqtt_client_publish(mqtt->mgr, mqtt->topic, str);
+    mqtt_client_publish(mqtt->mqc, mqtt->topic, str);
 }
 
 static void print_mqtt_double(data_output_t *output, double data, char const *format)
@@ -388,7 +386,7 @@ static void print_mqtt_double(data_output_t *output, double data, char const *fo
     char str[20];
     // use scientific notation for very big/small values
     if (data > 1e7 || data < 1e-4) {
-        int ret = snprintf(str, 20, "%g", data);
+        snprintf(str, 20, "%g", data);
     }
     else {
         int ret = snprintf(str, 20, "%.5f", data);
@@ -405,20 +403,10 @@ static void print_mqtt_double(data_output_t *output, double data, char const *fo
 static void print_mqtt_int(data_output_t *output, int data, char const *format)
 {
     char str[20];
-    int ret = snprintf(str, 20, "%d", data);
+    snprintf(str, 20, "%d", data);
     print_mqtt_string(output, str, format);
 }
 
-static void data_output_mqtt_poll(data_output_t *output)
-{
-    data_output_mqtt_t *mqtt = (data_output_mqtt_t *)output;
-
-    if (!mqtt)
-        return;
-
-    mqtt_client_poll(mqtt->mgr);
-}
-
 static void data_output_mqtt_free(data_output_t *output)
 {
     data_output_mqtt_t *mqtt = (data_output_mqtt_t *)output;
@@ -432,7 +420,8 @@ static void data_output_mqtt_free(data_output_t *output)
     //free(mqtt->homie);
     //free(mqtt->hass);
 
-    mqtt_client_free(mqtt->mgr);
+    mqtt_client_free(mqtt->mqc);
+
     free(mqtt);
 }
 
@@ -457,7 +446,7 @@ static char *mqtt_topic_default(char const *topic, char const *base, char const
     return ret;
 }
 
-struct data_output *data_output_mqtt_create(char const *host, char const *port, char *opts, char const *dev_hint)
+struct data_output *data_output_mqtt_create(struct mg_mgr *mgr, char const *host, char const *port, char *opts, char const *dev_hint)
 {
     data_output_mqtt_t *mqtt = calloc(1, sizeof(data_output_mqtt_t));
     if (!mqtt)
@@ -551,10 +540,9 @@ struct data_output *data_output_mqtt_create(char const *host, char const *port,
     mqtt->output.print_string = print_mqtt_string;
     mqtt->output.print_double = print_mqtt_double;
     mqtt->output.print_int    = print_mqtt_int;
-    mqtt->output.output_poll  = data_output_mqtt_poll;
     mqtt->output.output_free  = data_output_mqtt_free;
 
-    mqtt->mgr = mqtt_client_init(host, port, user, pass, client_id, retain);
+    mqtt->mqc = mqtt_client_init(mgr, host, port, user, pass, client_id, retain);
 
     return &mqtt->output;
 }
