@@ -17,6 +17,7 @@
 #include <math.h>
 
 #include "r_api.h"
+#include "r_util.h"
 #include "rtl_433.h"
 #include "r_private.h"
 #include "r_device.h"
@@ -29,8 +30,10 @@
 #include "output_mqtt.h"
 #include "output_influx.h"
 #include "write_sigrok.h"
+#include "mongoose.h"
 #include "compat_time.h"
 #include "fatal.h"
+#include "http_server.h"
 
 #ifdef _WIN32
 #include <io.h>
@@ -73,6 +76,54 @@ char const *version_string(void)
             ;
 }
 
+/* helper */
+
+struct mg_mgr *get_mgr(r_cfg_t *cfg)
+{
+    if (!cfg->mgr) {
+        cfg->mgr = calloc(1, sizeof(*cfg->mgr));
+        if (!cfg->mgr)
+            FATAL_CALLOC("get_mgr()");
+        mg_mgr_init(cfg->mgr, NULL);
+    }
+
+    return cfg->mgr;
+}
+
+void set_center_freq(r_cfg_t *cfg, uint32_t center_freq)
+{
+    cfg->frequencies = 1;
+    cfg->frequency_index = 0;
+    cfg->frequency[0] = center_freq;
+    sdr_set_center_freq(cfg->dev, center_freq, 0);
+}
+
+void set_freq_correction(r_cfg_t *cfg, int freq_correction)
+{
+    cfg->ppm_error = freq_correction;
+    sdr_set_freq_correction(cfg->dev, freq_correction, 0);
+}
+
+void set_sample_rate(r_cfg_t *cfg, uint32_t sample_rate)
+{
+    cfg->samp_rate = sample_rate;
+    sdr_set_sample_rate(cfg->dev, sample_rate, 0);
+}
+
+void set_gain_str(struct r_cfg *cfg, char const *gain_str)
+{
+    free(cfg->gain_str);
+    if (!gain_str) {
+        cfg->gain_str = NULL; // auto gain
+    }
+    else {
+        cfg->gain_str = strdup(gain_str);
+        if (!cfg->gain_str)
+            WARN_STRDUP("set_gain_str()");
+    }
+    sdr_set_tuner_gain(cfg->dev, gain_str, 0);
+}
+
 /* general */
 
 void r_init_cfg(r_cfg_t *cfg)
@@ -89,7 +140,11 @@ void r_init_cfg(r_cfg_t *cfg)
     if (!cfg->demod)
         FATAL_CALLOC("r_init_cfg()");
 
-    cfg->demod->level_limit = DEFAULT_LEVEL_LIMIT;
+    cfg->demod->level_limit = 0.0;
+    cfg->demod->min_level = -12.1442;
+    cfg->demod->min_snr = 9.0;
+
+    time(&cfg->frames_since);
 
     list_ensure_size(&cfg->demod->r_devs, 100);
     list_ensure_size(&cfg->demod->dumper, 32);
@@ -113,6 +168,8 @@ void r_free_cfg(r_cfg_t *cfg)
         sdr_close(cfg->dev);
     }
 
+    free(cfg->gain_str);
+
     for (void **iter = cfg->demod->dumper.elems; iter && *iter; ++iter) {
         file_info_t const *dumper = *iter;
         if (dumper->file && (dumper->file != stdout))
@@ -133,41 +190,14 @@ void r_free_cfg(r_cfg_t *cfg)
 
     list_free_elems(&cfg->in_files, NULL);
 
+    mg_mgr_free(cfg->mgr);
+    free(cfg->mgr);
+
     //free(cfg);
 }
 
 /* device decoder protocols */
 
-void update_protocol(r_cfg_t *cfg, r_device *r_dev)
-{
-    float samples_per_us = cfg->samp_rate / 1.0e6;
-
-    r_dev->f_short_width = r_dev->short_width > 0.0 ? 1.0 / (r_dev->short_width * samples_per_us) : 0;
-    r_dev->f_long_width  = r_dev->long_width > 0.0 ? 1.0 / (r_dev->long_width * samples_per_us) : 0;
-    r_dev->s_short_width = r_dev->short_width * samples_per_us;
-    r_dev->s_long_width  = r_dev->long_width * samples_per_us;
-    r_dev->s_reset_limit = r_dev->reset_limit * samples_per_us;
-    r_dev->s_gap_limit   = r_dev->gap_limit * samples_per_us;
-    r_dev->s_sync_width  = r_dev->sync_width * samples_per_us;
-    r_dev->s_tolerance   = r_dev->tolerance * samples_per_us;
-
-    // check for rounding to zero
-    if ((r_dev->short_width > 0 && r_dev->s_short_width <= 0)
-            || (r_dev->long_width > 0 && r_dev->s_long_width <= 0)
-            || (r_dev->reset_limit > 0 && r_dev->s_reset_limit <= 0)
-            || (r_dev->gap_limit > 0 && r_dev->s_gap_limit <= 0)
-            || (r_dev->sync_width > 0 && r_dev->s_sync_width <= 0)
-            || (r_dev->tolerance > 0 && r_dev->s_tolerance <= 0)) {
-        fprintf(stderr, "sample rate too low for protocol %u \"%s\"\n", r_dev->protocol_num, r_dev->name);
-        exit(1);
-    }
-
-    r_dev->verbose      = cfg->verbosity > 0 ? cfg->verbosity - 1 : 0;
-    r_dev->verbose_bits = cfg->verbose_bits;
-
-    r_dev->old_model_keys = cfg->old_model_keys; // TODO: temporary allow to change to new style model keys
-}
-
 void register_protocol(r_cfg_t *cfg, r_device *r_dev, char *arg)
 {
     r_device *p;
@@ -184,7 +214,10 @@ void register_protocol(r_cfg_t *cfg, r_device *r_dev, char *arg)
         *p = *r_dev; // copy
     }
 
-    update_protocol(cfg, p);
+    p->verbose      = cfg->verbosity > 0 ? cfg->verbosity - 1 : 0;
+    p->verbose_bits = cfg->verbose_bits;
+
+    p->old_model_keys = cfg->old_model_keys; // TODO: temporary allow to change to new style model keys
 
     p->output_fn  = data_acquired_handler;
     p->output_ctx = cfg;
@@ -224,14 +257,6 @@ void register_all_protocols(r_cfg_t *cfg, unsigned disabled)
     }
 }
 
-void update_protocols(r_cfg_t *cfg)
-{
-    for (void **iter = cfg->demod->r_devs.elems; iter && *iter; ++iter) {
-        r_device *r_dev = *iter;
-        update_protocol(cfg, r_dev);
-    }
-}
-
 /* output helper */
 
 void calc_rssi_snr(r_cfg_t *cfg, pulse_data_t *pulse_data)
@@ -244,7 +269,7 @@ void calc_rssi_snr(r_cfg_t *cfg, pulse_data_t *pulse_data)
     pulse_data->freq1_hz = (foffs1 + cfg->center_frequency);
     pulse_data->freq2_hz = (foffs2 + cfg->center_frequency);
     // NOTE: for (CU8) amplitude is 10x (because it's squares)
-    if (cfg->demod->sample_size == 1) { // amplitude (CU8)
+    if (cfg->demod->sample_size == 1 && !cfg->demod->use_mag_est) { // amplitude (CU8)
         pulse_data->rssi_db = 10.0f * log10f(ook_high_estimate) - 42.1442f; // 10*log10f(16384.0f)
         pulse_data->noise_db = 10.0f * log10f(ook_low_estimate) - 42.1442f; // 10*log10f(16384.0f)
         pulse_data->snr_db  = 10.0f * log10f(asnr);
@@ -301,8 +326,8 @@ char const **well_known_output_fields(r_cfg_t *cfg)
 
     if (cfg->verbose_bits)
         *p++ = "bits";
-    if (cfg->output_tag)
-        *p++ = "tag";
+    if (cfg->output_key)
+        *p++ = cfg->output_key;
     if (cfg->report_protocol)
         *p++ = "protocol";
     if (cfg->report_description)
@@ -413,12 +438,13 @@ int run_ook_demods(list_t *r_devs, pulse_data_t *pulse_data)
         case OOK_PULSE_PWM_OSV1:
             p_events += pulse_demod_osv1(pulse_data, r_dev);
             break;
+        case OOK_PULSE_NRZS:
+            p_events += pulse_demod_nrzs(pulse_data, r_dev);
+            break;
         // FSK decoders
         case FSK_PULSE_PCM:
         case FSK_PULSE_PWM:
-            break;
         case FSK_PULSE_MANCHESTER_ZEROBIT:
-            p_events += pulse_demod_manchester_zerobit(pulse_data, r_dev);
             break;
         default:
             fprintf(stderr, "Unknown modulation %u in protocol!\n", r_dev->modulation);
@@ -444,6 +470,7 @@ int run_fsk_demods(list_t *r_devs, pulse_data_t *fsk_pulse_data)
         case OOK_PULSE_PIWM_DC:
         case OOK_PULSE_DMC:
         case OOK_PULSE_PWM_OSV1:
+        case OOK_PULSE_NRZS:
             break;
         case FSK_PULSE_PCM:
             p_events += pulse_demod_pcm(fsk_pulse_data, r_dev);
@@ -487,6 +514,22 @@ void data_acquired_handler(r_device *r_dev, data_t *data)
 {
     r_cfg_t *cfg = r_dev->output_ctx;
 
+#ifndef NDEBUG
+    // check for undeclared csv fields
+    for (data_t *d = data; d; d = d->next) {
+        int found = 0;
+        for (char **p = r_dev->fields; *p; ++p) {
+            if (!strcmp(d->key, *p)) {
+                found = 1;
+                break;
+            }
+        }
+        if (!found) {
+            fprintf(stderr, "WARNING: Undeclared field \"%s\" in [%u] \"%s\"\n", d->key, r_dev->protocol_num, r_dev->name);
+        }
+    }
+#endif
+
     // replace textual battery key with numerical battery key
     if (!cfg->old_model_keys) {
         for (data_t *d = data; d; d = d->next) {
@@ -709,7 +752,7 @@ void data_acquired_handler(r_device *r_dev, data_t *data)
             output_tag = file_basename(cfg->in_filename);
         }
         data = data_prepend(data,
-                "tag", "Tag", DATA_STRING, output_tag,
+                cfg->output_key, "", DATA_STRING, output_tag,
                 NULL);
     }
 
@@ -735,6 +778,7 @@ data_t *create_report_data(r_cfg_t *cfg, int level)
             continue;
         if (level <= 0)
             continue;
+
         data = data_make(
                 "device",       "", DATA_INT, r_dev->protocol_num,
                 "name",         "", DATA_STRING, r_dev->name,
@@ -773,8 +817,12 @@ data_t *create_report_data(r_cfg_t *cfg, int level)
             "events",           "", DATA_INT, cfg->frames_events,
             NULL);
 
+    char since_str[LOCAL_TIME_BUFLEN];
+    format_time_str(since_str, "%Y-%m-%dT%H:%M:%S", cfg->report_time_tz, cfg->frames_since);
+
     data = data_make(
             "enabled",          "", DATA_INT, r_devs->len,
+            "since",            "", DATA_STRING, since_str,
             "frames",           "", DATA_DATA, data,
             "stats",            "", DATA_ARRAY, data_array(dev_data_list.len, DATA_DATA, dev_data_list.elems),
             NULL);
@@ -787,6 +835,7 @@ void flush_report_data(r_cfg_t *cfg)
 {
     list_t *r_devs = &cfg->demod->r_devs;
 
+    time(&cfg->frames_since);
     cfg->frames_count = 0;
     cfg->frames_fsk = 0;
     cfg->frames_events = 0;
@@ -855,12 +904,12 @@ void add_mqtt_output(r_cfg_t *cfg, char *param)
     char *opts = hostport_param(param, &host, &port);
     fprintf(stderr, "Publishing MQTT data to %s port %s\n", host, port);
 
-    list_push(&cfg->output_handler, data_output_mqtt_create(host, port, opts, cfg->dev_query));
+    list_push(&cfg->output_handler, data_output_mqtt_create(get_mgr(cfg), host, port, opts, cfg->dev_query));
 }
 
 void add_influx_output(r_cfg_t *cfg, char *param)
 {
-    list_push(&cfg->output_handler, data_output_influx_create(param));
+    list_push(&cfg->output_handler, data_output_influx_create(get_mgr(cfg), param));
 }
 
 void add_syslog_output(r_cfg_t *cfg, char *param)
@@ -873,9 +922,19 @@ void add_syslog_output(r_cfg_t *cfg, char *param)
     list_push(&cfg->output_handler, data_output_syslog_create(host, port));
 }
 
+void add_http_output(r_cfg_t *cfg, char *param)
+{
+    char *host = "0.0.0.0";
+    char *port = "8433";
+    hostport_param(param, &host, &port);
+    fprintf(stderr, "HTTP server at %s port %s\n", host, port);
+
+    list_push(&cfg->output_handler, data_output_http_create(get_mgr(cfg), host, port, cfg));
+}
+
 void add_null_output(r_cfg_t *cfg, char *param)
 {
-    (void)param;
+    UNUSED(param);
     list_push(&cfg->output_handler, NULL);
 }
 
@@ -893,6 +952,14 @@ void add_sr_dumper(r_cfg_t *cfg, char const *spec, int overwrite)
 
 void close_dumpers(struct r_cfg *cfg)
 {
+    for (void **iter = cfg->demod->dumper.elems; iter && *iter; ++iter) {
+        file_info_t *dumper = *iter;
+        if (dumper->file && (dumper->file != stdout)) {
+            fclose(dumper->file);
+            dumper->file = NULL;
+        }
+    }
+
     char const *labels[] = {
             "FRAME", // probe1
             "ASK", // probe2
