@@ -35,9 +35,11 @@
 #include "r_api.h"
 #include "sdr.h"
 #include "baseband.h"
+#include "pulse_analyzer.h"
 #include "pulse_detect.h"
 #include "pulse_detect_fsk.h"
 #include "pulse_demod.h"
+#include "rfraw.h"
 #include "data.h"
 #include "r_util.h"
 #include "optparse.h"
@@ -49,6 +51,7 @@
 #include "compat_paths.h"
 #include "fatal.h"
 #include "write_sigrok.h"
+#include "mongoose.h"
 
 #ifdef _WIN32
 #include <io.h>
@@ -67,6 +70,18 @@
 #include "getopt/getopt.h"
 #endif
 
+// note that Clang has _Noreturn but it's C11
+// #if defined(__clang__) ...
+#if !defined _Noreturn
+#if defined(__GNUC__)
+#define _Noreturn __attribute__((noreturn))
+#elif defined(_MSC_VER)
+#define _Noreturn __declspec(noreturn)
+#else
+#define _Noreturn
+#endif
+#endif
+
 r_device *flex_create_device(char *spec); // maybe put this in some header file?
 
 static void print_version(void)
@@ -75,6 +90,7 @@ static void print_version(void)
     fprintf(stderr, "Use -h for usage help and see https://triq.org/ for documentation.\n");
 }
 
+_Noreturn
 static void usage(int exit_code)
 {
     term_help_printf(
@@ -99,9 +115,7 @@ static void usage(int exit_code)
             "       Specify a negative number to disable a device decoding protocol (can be used multiple times)\n"
             "  [-G] Enable blacklisted device decoding protocols, for testing only.\n"
             "  [-X <spec> | help] Add a general purpose decoder (prepend -R 0 to disable all decoders)\n"
-            "  [-l <level>] Change detection level used to determine pulses (0-16384) (0=auto) (default: %i)\n"
-            "  [-z <value>] Override short value in data decoder\n"
-            "  [-x <value>] Override long value in data decoder\n"
+            "  [-Y level=<dB level>] Manual detection level used to determine pulses (-1.0 to -30.0) (0=auto)\n"
             "  [-n <value>] Specify number of samples to take (each sample is 2 bytes: 1 each of I & Q)\n"
             "  [-Y auto | classic | minmax] FSK pulse detector mode.\n"
             "\t\t= Analyze/Debug options =\n"
@@ -120,16 +134,17 @@ static void usage(int exit_code)
             "       Append output to file with :<filename> (e.g. -F csv:log.csv), defaults to stdout.\n"
             "       Specify host/port for syslog with e.g. -F syslog:127.0.0.1:1514\n"
             "  [-M time[:<options>] | protocol | level | stats | bits | help] Add various meta data to each output.\n"
-            "  [-K FILE | PATH | <tag>] Add an expanded token or fixed tag to every output line.\n"
+            "  [-K FILE | PATH | <tag> | <key>=<value>] Add an expanded token or fixed tag to every output line.\n"
             "  [-C native | si | customary] Convert units in decoded output.\n"
             "  [-T <seconds>] Specify number of seconds to run, also 12:34 or 1h23m45s\n"
             "  [-E hop | quit] Hop/Quit after outputting successful event(s)\n"
             "  [-h] Output this usage help and exit\n"
             "       Use -d, -g, -R, -X, -F, -M, -r, -w, or -W without argument for more help\n\n",
-            DEFAULT_FREQUENCY, DEFAULT_HOP_TIME, DEFAULT_SAMPLE_RATE, DEFAULT_LEVEL_LIMIT);
+            DEFAULT_FREQUENCY, DEFAULT_HOP_TIME, DEFAULT_SAMPLE_RATE);
     exit(exit_code);
 }
 
+_Noreturn
 static void help_protocols(r_device *devices, unsigned num_devices, int exit_code)
 {
     unsigned i;
@@ -147,6 +162,7 @@ static void help_protocols(r_device *devices, unsigned num_devices, int exit_cod
     exit(exit_code);
 }
 
+_Noreturn
 static void help_device(void)
 {
     term_help_printf(
@@ -172,6 +188,7 @@ static void help_device(void)
     exit(0);
 }
 
+_Noreturn
 static void help_gain(void)
 {
     term_help_printf(
@@ -183,6 +200,7 @@ static void help_gain(void)
     exit(0);
 }
 
+_Noreturn
 static void help_output(void)
 {
     term_help_printf(
@@ -208,6 +226,7 @@ static void help_output(void)
     exit(0);
 }
 
+_Noreturn
 static void help_meta(void)
 {
     term_help_printf(
@@ -232,6 +251,7 @@ static void help_meta(void)
     exit(0);
 }
 
+_Noreturn
 static void help_read(void)
 {
     term_help_printf(
@@ -253,6 +273,7 @@ static void help_read(void)
     exit(0);
 }
 
+_Noreturn
 static void help_write(void)
 {
     term_help_printf(
@@ -278,17 +299,9 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
     char time_str[LOCAL_TIME_BUFLEN];
     unsigned long n_samples;
 
-    for (size_t i = 0; i < cfg->output_handler.len; ++i) { // list might contain NULLs
-        data_output_poll(cfg->output_handler.elems[i]);
-    }
-
-    if (cfg->do_exit || cfg->do_exit_async)
-        return;
-
     if ((cfg->bytes_to_read > 0) && (cfg->bytes_to_read <= len)) {
         len = cfg->bytes_to_read;
-        cfg->do_exit = 1;
-        sdr_stop(cfg->dev);
+        cfg->exit_async = 1;
     }
 
     get_time_now(&demod->now);
@@ -318,9 +331,13 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
 
     // AM demodulation
     if (demod->sample_size == 1) { // CU8
-        envelope_detect(iq_buf, demod->buf.temp, n_samples);
-        //magnitude_true_cu8(iq_buf, demod->buf.temp, n_samples);
-        //magnitude_est_cu8(iq_buf, demod->buf.temp, n_samples);
+        if (demod->use_mag_est) {
+            //magnitude_true_cu8(iq_buf, demod->buf.temp, n_samples);
+            magnitude_est_cu8(iq_buf, demod->buf.temp, n_samples);
+        }
+        else { // amp est
+            envelope_detect(iq_buf, demod->buf.temp, n_samples);
+        }
     } else { // CS16
         //magnitude_true_cs16((int16_t *)iq_buf, demod->buf.temp, n_samples);
         magnitude_est_cs16((int16_t *)iq_buf, demod->buf.temp, n_samples);
@@ -366,7 +383,7 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
         }
         while (package_type) {
             int p_events = 0; // Sensor events successfully detected per package
-            package_type = pulse_detect_package(demod->pulse_detect, demod->am_buf, demod->buf.fm, n_samples, demod->level_limit, cfg->samp_rate, cfg->input_pos, &demod->pulse_data, &demod->fsk_pulse_data, fpdm);
+            package_type = pulse_detect_package(demod->pulse_detect, demod->am_buf, demod->buf.fm, n_samples, cfg->samp_rate, cfg->input_pos, &demod->pulse_data, &demod->fsk_pulse_data, fpdm);
             if (package_type) {
                 // new package: set a first frame start if we are not tracking one already
                 if (!demod->frame_start_ago)
@@ -548,7 +565,7 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
 
         if (fwrite(out_buf, 1, out_len, dumper->file) != out_len) {
             fprintf(stderr, "Short write, samples lost, exiting!\n");
-            sdr_stop(cfg->dev);
+            cfg->exit_async = 1;
         }
     }
 
@@ -557,14 +574,15 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
         cfg->bytes_to_read -= len;
 
     if (cfg->after_successful_events_flag && (d_events > 0)) {
-        if (cfg->after_successful_events_flag == 1) {
-            cfg->do_exit = 1;
-        }
-        cfg->do_exit_async = 1;
 #ifndef _WIN32
         alarm(0); // cancel the watchdog timer
 #endif
-        sdr_stop(cfg->dev);
+        if (cfg->after_successful_events_flag == 1) {
+            cfg->exit_async = 1;
+        }
+        else {
+            cfg->hop_now = 1;
+        }
     }
 
     time_t rawtime;
@@ -573,18 +591,16 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
     int hop_index = cfg->hop_times > cfg->frequency_index ? cfg->frequency_index : cfg->hop_times - 1;
     if (cfg->hop_times > 0 && cfg->frequencies > 1
             && difftime(rawtime, cfg->hop_start_time) > cfg->hop_time[hop_index]) {
-        cfg->do_exit_async = 1;
 #ifndef _WIN32
         alarm(0); // cancel the watchdog timer
 #endif
-        sdr_stop(cfg->dev);
+        cfg->hop_now = 1;
     }
     if (cfg->duration > 0 && rawtime >= cfg->stop_time) {
-        cfg->do_exit_async = cfg->do_exit = 1;
 #ifndef _WIN32
         alarm(0); // cancel the watchdog timer
 #endif
-        sdr_stop(cfg->dev);
+        cfg->exit_async = 1;
         fprintf(stderr, "Time expired, exiting!\n");
     }
     if (cfg->stats_now || (cfg->report_stats && cfg->stats_interval && rawtime >= cfg->stats_time)) {
@@ -595,6 +611,14 @@ static void sdr_callback(unsigned char *iq_buf, uint32_t len, void *ctx)
         if (cfg->stats_now)
             cfg->stats_now--;
     }
+
+    if (cfg->hop_now && !cfg->exit_async) {
+        cfg->hop_now = 0;
+        time(&cfg->hop_start_time);
+        cfg->frequency_index  = (cfg->frequency_index + 1) % cfg->frequencies;
+        cfg->center_frequency = cfg->frequency[cfg->frequency_index];
+        sdr_set_center_freq(cfg->dev, cfg->center_frequency, 0);
+    }
 }
 
 static int hasopt(int test, int argc, char *argv[], char const *optstring)
@@ -642,7 +666,7 @@ static struct conf_keywords const conf_keywords[] = {
         {"signal_grabber", 'S'},
         {"override_short", 'z'},
         {"override_long", 'x'},
-        {"fsk_pulse_detect_mode", 'Y'},
+        {"pulse_detect", 'Y'},
         {"output", 'F'},
         {"output_tag", 'K'},
         {"convert", 'C'},
@@ -761,7 +785,10 @@ static void parse_conf_option(r_cfg_t *cfg, int opt, char *arg)
         if (!arg)
             help_gain();
 
-        cfg->gain_str = arg;
+        free(cfg->gain_str);
+        cfg->gain_str = strdup(arg);
+        if (!cfg->gain_str)
+            FATAL_STRDUP("parse_conf_option()");
         break;
     case 'G':
         if (atobv(arg, 1) == 4) {
@@ -784,7 +811,11 @@ static void parse_conf_option(r_cfg_t *cfg, int opt, char *arg)
         cfg->out_block_size = atouint32_metric(arg, "-b: ");
         break;
     case 'l':
-        cfg->demod->level_limit = atouint32_metric(arg, "-l: ");
+        n = 1000;
+        if (arg && atoi(arg) > 0)
+            n = atoi(arg);
+        fprintf(stderr, "\n\tLevel limit has changed from \"-l %d\" to \"-Y level=%.1f\" in dB.\n\n", n, AMP_TO_DB(n));
+        exit(1);
         break;
     case 'n':
         cfg->bytes_to_read = atouint32_metric(arg, "-n: ") * 2;
@@ -930,16 +961,10 @@ static void parse_conf_option(r_cfg_t *cfg, int opt, char *arg)
         fprintf(stderr, "debug option (-D) is deprecated. See -v to increase verbosity\n");
         break;
     case 'z':
-        if (!arg)
-            usage(1);
-        if (cfg->demod->am_analyze)
-            cfg->demod->am_analyze->override_short = atoi(arg);
+        fprintf(stderr, "override_short (-z) is deprecated.\n");
         break;
     case 'x':
-        if (!arg)
-            usage(1);
-        if (cfg->demod->am_analyze)
-            cfg->demod->am_analyze->override_long = atoi(arg);
+        fprintf(stderr, "override_long (-x) is deprecated.\n");
         break;
     case 'R':
         if (!arg)
@@ -997,13 +1022,15 @@ static void parse_conf_option(r_cfg_t *cfg, int opt, char *arg)
         else if (strncmp(arg, "mqtt", 4) == 0) {
             add_mqtt_output(cfg, arg_param(arg));
         }
-        else if (strncmp(arg, "http", 4) == 0
-                || strncmp(arg, "influx", 6) == 0) {
+        else if (strncmp(arg, "influx", 6) == 0) {
             add_influx_output(cfg, arg);
         }
         else if (strncmp(arg, "syslog", 6) == 0) {
             add_syslog_output(cfg, arg_param(arg));
         }
+        else if (strncmp(optarg, "http", 4) == 0) {
+            add_http_output(cfg, arg_param(optarg));
+        }
         else if (strncmp(arg, "null", 4) == 0) {
             add_null_output(cfg, arg_param(arg));
         }
@@ -1014,6 +1041,11 @@ static void parse_conf_option(r_cfg_t *cfg, int opt, char *arg)
         break;
     case 'K':
         cfg->output_tag = arg;
+        cfg->output_key = asepc(&cfg->output_tag, '=');
+        if (!cfg->output_tag) {
+            cfg->output_tag = cfg->output_key;
+            cfg->output_key = "tag";
+        }
         break;
     case 'C':
         if (!arg)
@@ -1048,18 +1080,31 @@ static void parse_conf_option(r_cfg_t *cfg, int opt, char *arg)
     case 'Y':
         if (!arg)
             usage(1);
-        if (strcmp(arg, "auto") == 0) {
-            cfg->fsk_pulse_detect_mode = FSK_PULSE_DETECT_AUTO;
-        }
-        else if (strcmp(arg, "classic") == 0) {
-            cfg->fsk_pulse_detect_mode = FSK_PULSE_DETECT_OLD;
-        }
-        else if (strcmp(arg, "minmax") == 0) {
-            cfg->fsk_pulse_detect_mode = FSK_PULSE_DETECT_NEW;
-        }
-        else {
-            fprintf(stderr, "Invalid FSK pulse detector mode: %s\n", arg);
-            usage(1);
+        char *p = arg;
+        while (p && *p) {
+            if (!strncmp(p, "auto", 4))
+                cfg->fsk_pulse_detect_mode = FSK_PULSE_DETECT_AUTO;
+            else if (!strncmp(p, "classic", 7))
+                cfg->fsk_pulse_detect_mode = FSK_PULSE_DETECT_OLD;
+            else if (!strncmp(p, "minmax", 6))
+                cfg->fsk_pulse_detect_mode = FSK_PULSE_DETECT_NEW;
+            else if (!strncmp(p, "ampest", 6))
+                cfg->demod->use_mag_est = 0;
+            else if (!strncmp(p, "verbose", 7))
+                cfg->demod->detect_verbosity++;
+            else if (!strncmp(p, "magest", 6))
+                cfg->demod->use_mag_est = 1;
+            else if (!strncasecmp(p, "level", 5))
+                cfg->demod->level_limit = arg_float(p + 5, "-Y level: ");
+            else if (!strncasecmp(p, "minlevel", 8))
+                cfg->demod->min_level = arg_float(p + 8, "-Y minlevel: ");
+            else if (!strncasecmp(p, "minsnr", 6))
+                cfg->demod->min_snr = arg_float(p + 6, "-Y minsnr: ");
+            else {
+                fprintf(stderr, "Unknown pulse detector setting: %s\n", p);
+                usage(1);
+            }
+            p = arg_param(p);
         }
         break;
     case 'E':
@@ -1092,14 +1137,12 @@ sighandler(int signum)
 {
     if (CTRL_C_EVENT == signum) {
         fprintf(stderr, "Signal caught, exiting!\n");
-        g_cfg.do_exit = 1;
-        sdr_stop(g_cfg.dev);
+        g_cfg.exit_async = 1;
         return TRUE;
     }
     else if (CTRL_BREAK_EVENT == signum) {
         fprintf(stderr, "CTRL-BREAK detected, hopping to next frequency (-f). Use CTRL-C to quit.\n");
-        g_cfg.do_exit_async = 1;
-        sdr_stop(g_cfg.dev);
+        g_cfg.hop_now = 1;
         return TRUE;
     }
     return FALSE;
@@ -1115,21 +1158,67 @@ static void sighandler(int signum)
         return;
     }
     else if (signum == SIGUSR1) {
-        g_cfg.do_exit_async = 1;
-        sdr_stop(g_cfg.dev);
+        g_cfg.hop_now = 1;
         return;
     }
     else if (signum == SIGALRM) {
         fprintf(stderr, "Async read stalled, exiting!\n");
+        g_cfg.exit_code = 3;
     }
     else {
         fprintf(stderr, "Signal caught, exiting!\n");
     }
-    g_cfg.do_exit = 1;
-    sdr_stop(g_cfg.dev);
+    g_cfg.exit_async = 1;
 }
 #endif
 
+static void sdr_handler(sdr_event_t *ev, void *ctx)
+{
+    r_cfg_t *cfg = ctx;
+
+    data_t *data = NULL;
+    if (ev->ev & SDR_EV_RATE) {
+        data = data_append(data,
+                "sample_rate", "", DATA_INT, ev->sample_rate,
+                NULL);
+    }
+    if (ev->ev & SDR_EV_CORR) {
+        data = data_append(data,
+                "freq_correction", "", DATA_INT, ev->freq_correction,
+                NULL);
+    }
+    if (ev->ev & SDR_EV_FREQ) {
+        data = data_append(data,
+                "center_frequency", "", DATA_INT, ev->center_frequency,
+                "frequencies", "", DATA_COND, cfg->frequencies > 1, DATA_ARRAY, data_array(cfg->frequencies, DATA_INT, cfg->frequency),
+                "hop_times", "", DATA_COND, cfg->frequencies > 1, DATA_ARRAY, data_array(cfg->hop_times, DATA_INT, cfg->hop_time),
+                NULL);
+    }
+    if (ev->ev & SDR_EV_GAIN) {
+        data = data_append(data,
+                "gain", "", DATA_STRING, ev->gain_str,
+                NULL);
+    }
+    if (data) {
+        for (size_t i = 0; i < cfg->output_handler.len; ++i) { // list might contain NULLs
+            data_output_print(cfg->output_handler.elems[i], data);
+        }
+        data_free(data);
+    }
+
+    if (ev->ev == SDR_EV_DATA) {
+        if (cfg->mgr) {
+            mg_mgr_poll(cfg->mgr, 0);
+        }
+
+        if (!cfg->exit_async)
+            sdr_callback((unsigned char *)ev->buf, ev->len, ctx);
+    }
+
+    if (cfg->exit_async)
+        sdr_stop(cfg->dev);
+}
+
 int main(int argc, char **argv) {
 #ifndef _WIN32
     struct sigaction sigact;
@@ -1184,8 +1273,10 @@ int main(int argc, char **argv) {
         add_infile(cfg, argv[optind++]);
     }
 
+    pulse_detect_set_levels(demod->pulse_detect, demod->use_mag_est, demod->level_limit, demod->min_level, demod->min_snr, demod->detect_verbosity);
+
     if (demod->am_analyze) {
-        demod->am_analyze->level_limit = &demod->level_limit;
+        demod->am_analyze->level_limit = DB_TO_AMP(demod->level_limit);
         demod->am_analyze->frequency   = &cfg->center_frequency;
         demod->am_analyze->samp_rate   = &cfg->samp_rate;
         demod->am_analyze->sample_size = &demod->sample_size;
@@ -1221,8 +1312,6 @@ int main(int argc, char **argv) {
     // register default decoders if nothing is configured
     if (!cfg->no_default_devices) {
         register_all_protocols(cfg, 0); // register all defaults
-    } else {
-        update_protocols(cfg);
     }
 
     // check if we need FM demod
@@ -1314,10 +1403,29 @@ int main(int argc, char **argv) {
                 }
                 if (cfg->verbosity)
                     fprintf(stderr, "Verifying test data with device %s.\n", r_dev->name);
+                if (rfraw_check(e)) {
+                    pulse_data_t pulse_data = {0};
+                    rfraw_parse(&pulse_data, e);
+                    list_t single_dev = {0};
+                    list_push(&single_dev, r_dev);
+                    if (!pulse_data.fsk_f2_est)
+                        r += run_ook_demods(&single_dev, &pulse_data);
+                    else
+                        r += run_fsk_demods(&single_dev, &pulse_data);
+                    list_free_elems(&single_dev, NULL);
+                } else
                 r += pulse_demod_string(e, r_dev);
                 continue;
             }
             // otherwise test all decoders
+            if (rfraw_check(line)) {
+                pulse_data_t pulse_data = {0};
+                rfraw_parse(&pulse_data, line);
+                if (!pulse_data.fsk_f2_est)
+                    r += run_ook_demods(&demod->r_devs, &pulse_data);
+                else
+                    r += run_fsk_demods(&demod->r_devs, &pulse_data);
+            } else
             for (void **iter = demod->r_devs.elems; iter && *iter; ++iter) {
                 r_device *r_dev = *iter;
                 if (cfg->verbosity)
@@ -1336,6 +1444,14 @@ int main(int argc, char **argv) {
     // Special case for string test data
     if (cfg->test_data) {
         r = 0;
+        if (rfraw_check(cfg->test_data)) {
+            pulse_data_t pulse_data = {0};
+            rfraw_parse(&pulse_data, cfg->test_data);
+            if (!pulse_data.fsk_f2_est)
+                r += run_ook_demods(&demod->r_devs, &pulse_data);
+            else
+                r += run_fsk_demods(&demod->r_devs, &pulse_data);
+        } else
         for (void **iter = demod->r_devs.elems; iter && *iter; ++iter) {
             r_device *r_dev = *iter;
             if (cfg->verbosity)
@@ -1395,11 +1511,23 @@ int main(int argc, char **argv) {
 
             // special case for pulse data file-inputs
             if (demod->load_info.format == PULSE_OOK) {
-                while (!cfg->do_exit) {
+                while (!cfg->exit_async) {
                     pulse_data_load(in_file, &demod->pulse_data, cfg->samp_rate);
                     if (!demod->pulse_data.num_pulses)
                         break;
 
+                    for (void **iter2 = demod->dumper.elems; iter2 && *iter2; ++iter2) {
+                        file_info_t const *dumper = *iter2;
+                        if (dumper->format == VCD_LOGIC) {
+                            pulse_data_print_vcd(dumper->file, &demod->pulse_data, '\'');
+                        } else if (dumper->format == PULSE_OOK) {
+                            pulse_data_dump(dumper->file, &demod->pulse_data);
+                        } else {
+                            fprintf(stderr, "Dumper (%s) not supported on OOK input\n", dumper->spec);
+                            exit(1);
+                        }
+                    }
+
                     if (demod->pulse_data.fsk_f2_est) {
                         run_fsk_demods(&demod->r_devs, &demod->pulse_data);
                     }
@@ -1442,7 +1570,7 @@ int main(int argc, char **argv) {
                 demod->sample_file_pos = ((float)n_blocks * DEFAULT_BUF_LENGTH + n_read) / cfg->samp_rate / 2 / demod->sample_size;
                 n_blocks++; // this assumes n_read == DEFAULT_BUF_LENGTH
                 sdr_callback(test_mode_buf, n_read, cfg);
-            } while (n_read != 0 && !cfg->do_exit);
+            } while (n_read != 0 && !cfg->exit_async);
 
             // Call a last time with cleared samples to ensure EOP detection
             if (demod->sample_size == 1) { // CU8
@@ -1456,6 +1584,9 @@ int main(int argc, char **argv) {
             }
             demod->sample_file_pos = ((float)n_blocks + 1) * DEFAULT_BUF_LENGTH / cfg->samp_rate / 2 / demod->sample_size;
             sdr_callback(test_mode_buf, DEFAULT_BUF_LENGTH, cfg);
+#ifndef _WIN32
+            alarm(0); // cancel the watchdog timer
+#endif
 
             //Always classify a signal at the end of the file
             if (demod->am_analyze)
@@ -1483,8 +1614,9 @@ int main(int argc, char **argv) {
     // Normal case, no test data, no in files
     r = sdr_open(&cfg->dev, &demod->sample_size, cfg->dev_query, cfg->verbosity);
     if (r < 0) {
-        exit(1);
+        exit(2);
     }
+    cfg->dev_info = sdr_get_dev_info(cfg->dev);
 
 #ifndef _WIN32
     sigact.sa_handler = sighandler;
@@ -1502,8 +1634,8 @@ int main(int argc, char **argv) {
     /* Set the sample rate */
     r = sdr_set_sample_rate(cfg->dev, cfg->samp_rate, 1); // always verbose
 
-    if (cfg->verbosity || demod->level_limit)
-        fprintf(stderr, "Bit detection level set to %d%s.\n", demod->level_limit, (demod->level_limit ? "" : " (Auto)"));
+    if (cfg->verbosity || demod->level_limit < 0.0)
+        fprintf(stderr, "Bit detection level set to %.1f%s.\n", demod->level_limit, (demod->level_limit < 0.0 ? "" : " (Auto)"));
 
     r = sdr_apply_settings(cfg->dev, cfg->settings_str, 1); // always verbose for soapy
 
@@ -1534,45 +1666,35 @@ int main(int argc, char **argv) {
         cfg->stop_time += cfg->duration;
     }
 
-    uint32_t samp_rate = cfg->samp_rate;
-    while (!cfg->do_exit) {
+    cfg->center_frequency = cfg->frequency[cfg->frequency_index];
+    r = sdr_set_center_freq(cfg->dev, cfg->center_frequency, 1); // always verbose
+
         time(&cfg->hop_start_time);
-
-        /* Set the cfg->frequency */
-        cfg->center_frequency = cfg->frequency[cfg->frequency_index];
-        r = sdr_set_center_freq(cfg->dev, cfg->center_frequency, 1); // always verbose
-
-        if (samp_rate != cfg->samp_rate) {
-            r = sdr_set_sample_rate(cfg->dev, cfg->samp_rate, 1); // always verbose
-            update_protocols(cfg);
-            samp_rate = cfg->samp_rate;
-        }
-
 #ifndef _WIN32
         signal(SIGALRM, sighandler);
         alarm(3); // require callback to run every 3 second, abort otherwise
 #endif
-        r = sdr_start(cfg->dev, sdr_callback, (void *)cfg,
+        r = sdr_start(cfg->dev, sdr_handler, (void *)cfg,
                 DEFAULT_ASYNC_BUF_NUMBER, cfg->out_block_size);
         if (r < 0) {
             fprintf(stderr, "WARNING: async read failed (%i).\n", r);
-            break;
         }
 #ifndef _WIN32
         alarm(0); // cancel the watchdog timer
 #endif
-        cfg->do_exit_async = 0;
-        cfg->frequency_index = (cfg->frequency_index + 1) % cfg->frequencies;
-    }
 
     if (cfg->report_stats > 0) {
         event_occurred_handler(cfg, create_report_data(cfg, cfg->report_stats));
         flush_report_data(cfg);
     }
 
-    if (!cfg->do_exit)
+    if (!cfg->exit_async) {
         fprintf(stderr, "\nLibrary error %d, exiting...\n", r);
+        cfg->exit_code = r;
+    }
 
+    if (cfg->exit_code >= 0)
+        r = cfg->exit_code;
     r_free_cfg(cfg);
 
     return r >= 0 ? r : -r;
